/* You will need the Paradox Engine to compile this program. If you have   */
/* placed directory for Engine in your INCLUDE path and LIBRARY path, the  */
/* modules should compile fine with: tcc -ml mci parse send pxengtcl.lib   */
/* Otherwise, specify where Engine header file and library files are       */
/* using the -I and -L compiler switches                                    */

#define MCIMAIN

#include <stdio.h>
#include <stdlib.h>
#include <pxengine.h>
#include "mci.h"

void closeTables(void);
void getFldNames(void);
void makeTblExist(char *tblName,int nFlds,char **fld, char ** types,int keyFields);
void openTables(void);
void shutdownEngine(void);
void startEngine(void);

/**** DEFINE MESSAGE TABLE INFORMATION **** /
char *msgFields[] =
{
  "Message#","Line#","Text"
};
char "msgTypes[] =
{
  "N","N","A80"
};
int msgKeyFields = 2;
#define msgNFields sizeof(msgTypes)/sizeof(char*)

/ **** DEFINE ROUTE TABLE INFORMATION **** /
char *routeFields[] =
{
  "Message#","TOorCC","User Name","User ID"
};
char *routeTypes[] =
{
  "N","A5","A20","A8"
};
int routeKeyFields = 3;
#define routeNFields sizeof(routeTypes)/sizeof(char*)

int lastPxErr;
FIELDHANDLE fh[] = {1};
TABLEHANDLE headerTbl,msgTbl,routeTbl;

int main(int argc,char *argv[])
{
        startEngine();
        openTables();
        getFldNames();
        if (argc > 1)
             parse(argv[1]);    /*only add to tables if filename passed in */
        send();
        closeTables();
        shutdownEngine();
        return 0;
}

void startEngine(void)
{
        PXCheck(0x101,PXInit());
}

void shutdownEngine(void)
{
        PXCheck(0x110,PXExit());
}

void openTables(void)
{

makeTblExist("Header",headerNFields,headerFields,headerTypes,HeaderKeyFields);
makeTblExist("Message",msgNFields,msgFields,msgTypes,msgKeyFields);
makeTblExist("Route",routeNFields,routeFields,routeTypes,routeKeyFields);
       PXCheck(0x120,PXTblOpen("Header",&headerTbl,0,0));
       PXCheck(0x121,PXTblOpen("Message",&msgTbl,0,0));
       PXCheck(0x122,PXTblOpen("Route",&routeTbl,0,0));
       PXCheck(0x123,PXRecBuffOpen(headerTbl,&headerRec));
       PXCheck(0x125,PXRecBufOpen(msgTbl,&msgRec));
       PXCheck(0x126,PXRecBufOpen(routeTbl,&routeRec));
}
void closeTables(void)
{
        PXCheck(0x130,PxTblClose(headerTbl));
        PXCheck(0x131,PXTblClose(msgTbl));
        PXCheck(0x132,PXTblClose(routeTbl));
}

void getFldNames(void)
{
        PXCheck(0x140,PXFldHandle(headerTbl,"Date Received",&hdrDateFld));
        PXCheck(0x141,PXFldHandle(headerTbl,"Message#",&hdrMsgFld));
        PXCheck(0x142,PXFldHandle(headerTbl,"Subject",&hdrSubjectFld));
        PXCheck(0x143,PXFldHandle(headerTbl,"From User",&hdrUserFld));
        PXCheck(0x144,PXFldHandle(headerTbl,"From ID",&hdrIDFld));
        PXCheck(0x145,PXFldHandle(msgTbl,"MEssage#",&msfMsgNumFld));
        PXCheck(0x146,PXFldHandle(msgTbl,"Line#",&msgLineFld));
        PXCheck(0x147,PXFldHandle(msgTbl,"Text",&msfTextFld));
        PXCheck(0x148,PXFldHandle(routeTbl,"Message#",&routeMsgFld));
        PXCheck(0x149,PXFldHandle(routeTbl,"ToOrCC",&routeToFld));
        PXCheck(0x14A,PxFldHandle(routeTbl,"User Name",&routeUserFld));
        PXCheck(0x14B,PxFldHandle(routeTbl,"User ID",&routeIDFld));
        return;
}

void makeTblExist(char *tblName,int nFlds,char **fld,char **types,int keyFields)
{
        int exist;
        PXCheck(0x150,PXTblExist(tblName,&exist));
        if (!exist)
        {
                PXCheck(0x151,PXTblCreate(tblName,nFlds,fld,types));
                if (keyFields)
                        PXCheck(0x152,PXKeyAdd(tblName,keyFields,fh,PRIMARY));
        }
        return;
}

void PXCheck(int loc,int errCode)
{
        lastPXErr=errCode;
        if (errCode==PXSUCCESS)
                 return;
        switch (loc)
        {
                case 0x234:
                case 0x237:
                case 0x247:
                case 0x353:
                case 0x254:      return;
                default: printf("Error '%s' at %04x\n",PXErrMsg(errCode),loc);
                                       exit (1);
        }
}

int PXLastErr(void)
{
        return lastPXErr;
}
